//
//   @file    news.cxx         
//   
//            builds the grid table for the downloads.htm file  
//          
//
//   @author M.Sutton
// 
//   Copyright (C) 2013 M.Sutton (sutt@cern.ch)    
//
//   $Id: gprint.cxx, v0.0   Wed 26 Jun 2013 23:05:02 CEST sutt $


#include <iostream>
// #include <ios>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include <cstdlib>

#include "applcolours.ihtml"

#include "grids.h"

#include <algorithm>


/// the real row class - has a "row" (not a tr) for each grid/data 
/// set combination



bool contains( const std::string& s, const std::string& regex ) { 
  return s.find(regex)!=std::string::npos;
}


std::string chop( const std::string& s, const std::string& regex ) { 
  std::string _s = s;
  if ( contains(_s,regex) ) _s.erase( 0, _s.find(regex)+regex.size() );
  return _s;
}





int intdate(const std::string& d) { 

  std::string year = d;
  year.erase(year.find("-"),year.size());

  std::string _month = chop(d,"-");
  _month.erase(_month.find("-"),_month.size());

  std::string _day = chop(d,"-");
  _day = chop(_day,"-");
  _day.erase(_day.find(":"), _day.size() );
  
  return std::atoi( (year+_month+_day).c_str() );

}




std::string dateparse(const std::string& d) { 
  std::string year = d;
  year.erase(year.find("-"),year.size());

  std::string _month = chop(d,"-");
  _month.erase(_month.find("-"),_month.size());

  std::string _day = chop(d,"-");
  _day = chop(_day,"-");
  _day.erase(_day.find(":"), _day.size() );
  
  static std::string smonth[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

  int month = std::atoi(_month.c_str());
  int day   = std::atoi(_day.c_str());


  std::stringstream s;
  if ( day<10 ) s << " ";
  s << day;

  if       ( day==1 || day==21 || day==31 ) s << "st ";
  else  if ( day==2 || day==22 )            s << "nd ";
  else  if ( day==3 || day==23 )            s << "rd ";
  else                                      s << "th ";
  
  s << smonth[month-1] << " " << year;

  return s.str();
}








class row { 

public:

  /// fill the fields of the row
  row( const std::string& d, const std::string& s ) : 
    m_datei(intdate(d)),
    m_date(d), m_text(s) 
  { }

  /// fill the fields of the row
  row( std::ostream& stream, const std::string& d, const std::string& s ) : 
    m_datei(intdate(d)),
    m_date(d), m_text(s) { 
    print(stream);
  }

  
  /// actually print out the row
  void print( std::ostream& stream=std::cout ) {

    //    stream = s;

    static std::string tc[2] = { TABCOLOUR1, TABCOLOUR2 }; 
    static int icol = 1;
    
    std::string colour = tc[icol%2]; icol++;

    //  stream << "<tr bgcolor=" << colour << ">" << std::endl;
    stream << "<tr>" << std::endl;
    stream << "<td width=20%  valign=\"top\" bgcolor=BGCOLOUR2 > " << dateparse(m_date) << "</td><td width=90%  valign=\"top\"  bgcolor=BGCOLOUR2> " << m_text << "</td>\n" << std::endl;
    stream << "</tr>" << std::endl;

  }

  bool operator<( const row& r)  const { return m_datei<r.m_datei; }
  bool operator<=( const row& r) const { return m_datei<=r.m_datei; }
  bool operator>=( const row& r) const { return m_datei>=r.m_datei; }
  bool operator>( const row& r)  const { return m_datei>r.m_datei; }

private:


  /// some basic table entry printout routines for an easier life 
  
  /// add a table column
  std::string tabcol(const std::string& s) { 
    //  std::cout << "  <td colspan=\"" << n << "\" >\n";
    std::string _stream = "  <td>\n   " + s + "\n  </td>\n";
    return _stream;
  }
  

private:


  int m_datei;

  std::string m_date;
  std::string m_text;

};








int print( std::vector<row>& rows, std::ostream& stream=std::cout ) {

  /// read all the rows from the data base

  stream << "<!--- emacs: this is -*- html -*- --->\n\n";
  stream << "#include \"applpre.ihtml\"\n\n";
  stream << "<font face=\"Arial, Geneva\" color=FONTCOLOUR2 size=\"4\"><b>New developments and releases</b></font>\n";	 
  stream << "<br><br>\n\n";
  
  stream << "<font face=\"Arial, Geneva\" color=FONTCOLOUR1 size=\"2\">\n\n";
  
  //  stream << "Here any new developments or releases are briefly summarised<p>\n"; 

  stream  << "<table id=\"newstable\" bgcolor=BGCOLOUR2 align=\"left\" border=\"0\" cellpadding=\"5em\" cellspacing=\"0\">\n";
 
  for ( int i=0 ; i<rows.size() ; i++ ) rows[i].print(stream);

  stream << "</table>\n";

  stream << "</font>\n\n";
  stream << "#include \"applpost.ihtml\"\n" << std::endl;
  
  return 0;

}







std::vector<row> parsefile( const std::string& filename ) { 

  std::fstream datfile;
  datfile.open(filename.c_str(), std::ios_base::in);


  char text[2056];

  std::string date;
  std::string body;

  std::vector<row> rows;

  while ( datfile.getline( text, 2056 ) ) { 
    
    std::string line( text );
    
    if ( contains( line, "date:" ) ) { 

      if ( body.size()>0 ) { 
	rows.push_back( row( date, body ) );
      }

      date = chop( line,"date: "); 

      body = "";
    }
    else { 
      if ( body.size() ) body += "<br>";
      body += line;
    }

    //    std::cout << text << std::endl;
  }
  
  if ( body.size()>0 )  rows.push_back( row( date, body ) );
  
  datfile.close();

  return rows;
}





int main() { 

  /// read in the dat file

  std::vector<row> rows = parsefile( "news.dat" );

  /// sort into reverse date order
  std::stable_sort( rows.begin(), rows.end() );
  std::reverse( rows.begin(), rows.end() );


  /// print to file

  //  std::fstream file;
  //  file.open("news.ihtml", std::ios_base::out);
  
  //  std::ostream& mystream = file;
  
  print( rows, std::cout );

  //  file.close();

  return 0;
}
